/*
 0x2D     Power control    2 (Measure mode)
 
 */


module adxl362_read
  ( input CLK, RST,

    output 	     CS_N,
    output reg 	     SCLK, 
    output 	     MOSI,
    input 	     MISO,

    input 	     GO,
    output 	     BUSY,
    output reg [7:0] X, Y, Z,
    output 	     STROBE );

   reg 		     INIT_DONE;
   
   parameter ClockCount = 16; // 1-5MHz, 32clock/period @ 100MHz = 3MHz
  // parameter ClockCount = 1;
   reg [7:0] 	 CLK_CNT;
   
   // Register 08: XDATA, 09: YDATA, 0a: ZDATA   

   reg [5:0] 	 STAT; // IDLE, CMD, ADR, DAT1, DAT2, DAT3
   reg [7:0] 	 BIT;
   
   wire 	 STAT_IDLE = STAT[5];
   wire 	 STAT_CMD = STAT[4];
   wire 	 STAT_ADR = STAT[3];
   
   reg 		 SCLK_R;

   wire 	 SCLK_NEGEDGE = SCLK_R & ~SCLK;
   wire 	 SCLK_POSEDGE = ~SCLK_R & SCLK;

   assign CS_N = STAT_IDLE;
   
   always @ (posedge CLK) begin
      SCLK_R <= SCLK;
      
      if (RST) begin
	 CLK_CNT <= 1;
	 SCLK <= 0;
      end else begin
	 if (STAT_IDLE) begin
	    CLK_CNT <= 1;
	 end else begin
	    CLK_CNT <= (CLK_CNT == ClockCount) ? 0 : CLK_CNT + 1;
	    if (CLK_CNT == 0) SCLK <= ~SCLK;
	 end
      end
      
   end


   always @ (posedge CLK) begin
      if (RST) begin
	 STAT <= 6'b100_000;
	 BIT <= 8'b1000_0000;
	 INIT_DONE <= 0;
      end else begin
	 if (SCLK_NEGEDGE) BIT <= { BIT[0], BIT[7:1] };

	 casex (STAT)
	   6'b100_000: begin // IDLE
	      if (GO | ~INIT_DONE) STAT <= { STAT[0], STAT[5:1] };
	   end

	   6'b0xx_xxx: begin // CMD, ADR, DAT123
	      if (SCLK_NEGEDGE & BIT[0]) begin
		 if (~INIT_DONE & STAT[2]) begin 
		    STAT <= 6'b100_000;
		    INIT_DONE <= 1;
		 end 
		 else 		 STAT <= { STAT[0], STAT[5:1] };
	      end
	   end
	 endcase
      end
	
   end


   wire [7:0] INIT_CMD_ADR = STAT_CMD ? 8'h0a : STAT_ADR ? 8'h2d : 8'h02;
   wire [7:0] CMD_ADR = STAT_CMD ? 8'h0b : STAT_ADR ? 8'h08 : 8'hx;
   
   assign MOSI = |((INIT_DONE ? CMD_ADR : INIT_CMD_ADR) & BIT);
   
   always @ (posedge CLK) begin
      if (GO) begin
	 X <= 0;
	 Y <= 0;
	 Z <= 0;
      end else begin
	 X <= X | ((STAT[2] & MISO) ? BIT : 0);
	 Y <= Y | ((STAT[1] & MISO) ? BIT : 0);
	 Z <= Z | ((STAT[0] & MISO) ? BIT : 0);
      end
	
   end

   assign STROBE = STAT[0] & BIT[0] & SCLK_NEGEDGE;

   assign BUSY = GO | ~STAT_IDLE;
   

endmodule // adxl362_read
