module top
  ( input CLK,
    input 	  RST,
     
    output 	  ADXL_CS_N,
    output 	  ADXL_SCLK, 
    output 	  ADXL_MOSI,
    input 	  ADXL_MISO,

    output 	  TMP_SCL, 
    inout 	  TMP_SDA,

    input 	  PS2_CLK, PS2_DAT,
  
    output [2:0]  TRI_LED1, TRI_LED2,

    output [15:0] LED,

    input [15:0] SW,
    output [7:0]  ANODE,
    output [7:0]  CATHODE );


   // 27bits: 1Hz, 24bits: 8Hz
   reg [23:0] 	  CNT;
   always @ (posedge CLK) 
     CNT <= RST ? 0 : CNT + 1;

   wire [31:0] 	  DISP_PS2_MOUSE, DISP_PS2_CNT, DISP_PS2,
		  DISP_ADXL, DISP_ADT, DISP_VAL;


   wire [2:0] 	  ACCEL_LED, MOUSE_LED;

   // --------------------------------------------------
   // 7 Segment Display
   
   assign DISP_VAL = SW[0] ? DISP_ADT :
		     SW[1] ? DISP_ADXL :
		     SW[2] ? DISP_PS2 :
		     SW[3] ? DISP_PS2_CNT :
		     SW[4] ? DISP_PS2_MOUSE :
		     0;
   
   segment_disp_ctrl sdc ( .CLK(CLK), .RST(RST), .VAL(DISP_VAL),
			   .ANODE(ANODE), .CATHODE(CATHODE) );
   

   // --------------------------------------------------
   // Tri-Color LED
   
   assign TRI_LED1 = SW[15] ? ACCEL_LED : 0;
   assign TRI_LED2 = SW[14] ? MOUSE_LED : 0;

   // --------------------------------------------------
   // PS/2 Mouse

   wire [15:0] 	  MOUSE_X, MOUSE_Y;
   wire 	  MOUSE_STROBE, MOUSE_L, MOUSE_R;
   reg [31:0] 	  MOUSE_POS;
   
   ps2mouse p2m ( .CLK(CLK), .RST(RST), .PS2_CLK(PS2_CLK), .PS2_DAT(PS2_DAT),
		  .STROBE(MOUSE_STROBE), .X(MOUSE_X), .Y(MOUSE_Y),
		  .LEFT(MOUSE_L), .RIGHT(MOUSE_R) );
   

   always @ (posedge CLK)
     MOUSE_POS <= { MOUSE_X, MOUSE_Y };
   
   assign DISP_PS2_MOUSE = MOUSE_POS;

   wire 	  MOUSE_LED_TEMP;
   duty_ctrl mb_dc ( .CLK(CLK), .DUTY(64), .Q(MOUSE_LED_TEMP) );

   assign MOUSE_LED = { (MOUSE_L ? MOUSE_LED_TEMP : 1'b0 ),
			(MOUSE_R ? MOUSE_LED_TEMP : 1'b0 ), 1'b0 };
 
      
   // --------------------------------------------------
   // PS/2

   wire 	  PS2_STROBE;
   wire [7:0] 	  PS2_CODE;

   ps2recv p2r ( .CLK(CLK), .RST(RST), .PS2_CLK(PS2_CLK), .PS2_DAT(PS2_DAT),
		 .STROBE(PS2_STROBE), .KEYCODE(PS2_CODE) );
   
   reg [31:0] 	  PS2_CODES, PS2_CNT;
   
   always @ (posedge CLK) begin
      if (RST) begin
	PS2_CODES <= 0;
	 PS2_CNT <= 0;
      end else begin
	 if (PS2_STROBE) begin
	   PS2_CODES <= {PS2_CODES[23:0], PS2_CODE};
	    PS2_CNT <= PS2_CNT + 1;
	 end
      end
   end // always @ (posedge CLK)
   
   assign DISP_PS2 = PS2_CODES;
   assign DISP_PS2_CNT = PS2_CNT;
      
   
   // --------------------------------------------------
   // Accelerometer
   
   wire [7:0] 	  ADXL_X, ADXL_Y, ADXL_Z;
   
   wire 	  ADXL_STROBE;
   wire 	  ADXL_GO = &CNT;

   adxl362_read adxl ( .CLK(CLK), .RST(RST), .GO(ADXL_GO),
		       .X(ADXL_X), .Y(ADXL_Y), .Z(ADXL_Z),
		       .SCLK(ADXL_SCLK), .CS_N(ADXL_CS_N),
		       .MOSI(ADXL_MOSI), .MISO(ADXL_MISO),
		       .STROBE(ADXL_STROBE) );

   reg [23:0] 	  ACCEL;

   always @ (posedge CLK)
     if (ADXL_STROBE) ACCEL <= {ADXL_X, ADXL_Y, ADXL_Z};

   assign DISP_ADXL = {8'b0, ACCEL};
   
   // --------------------------------------------------
   // Temperature Sensor
   
   wire [15:0] 	  ADT_OUT;
   wire [31:0] 	  ADTC_OUT;
   wire 	  ADT_STROBE, ADTC_STROBE;
   wire 	  ADT_GO = &CNT;
   
   reg [15:0] 	  TEMP;
   
   adt7420_read adt ( .CLK(CLK), .RST(RST), .GO(ADT_GO),
		      .DATA(ADT_OUT), .STROBE(ADT_STROBE),
		      .SCL(TMP_SCL), .SDA(TMP_SDA) );


   adt7420_conv_0 adtc (
			.ap_clk (CLK),
			.ap_rst (RST), 
			.ap_start (ADT_STROBE),
			.ap_done (ADTC_STROBE),
			.ap_ready(),
			.ap_return(ADTC_OUT),
			.raw({16'b0, ADT_OUT})
			);
   
   
   always @ (posedge CLK) 
     if (ADTC_STROBE) TEMP <= ADTC_OUT[15:0];

   assign DISP_ADT = {16'b0, TEMP};

   // --------------------------------------------------
   // Accelerometer to Tri-color LED, but not full brightness

   duty_ctrl acd1 ( .CLK(CLK), .DUTY({2'b0, ACCEL[23:18]}), .Q(ACCEL_LED[2]) );
   duty_ctrl acd2 ( .CLK(CLK), .DUTY({2'b0, ACCEL[15:10]}), .Q(ACCEL_LED[1]) );
   duty_ctrl acd3 ( .CLK(CLK), .DUTY({2'b0, ACCEL[ 7: 2]}), .Q(ACCEL_LED[0]) );

   // --------------------------------------------------
   // LED kurukuru with PWM

   reg [7:0] 	  DUTY[15:0];
   initial begin
      DUTY[15] <= 250;  DUTY[14] <= 235;  DUTY[13] <= 210;  DUTY[12] <= 195;
      DUTY[11] <= 180;  DUTY[10] <= 165;  DUTY[ 9] <= 150;  DUTY[ 8] <= 135;
      DUTY[ 7] <= 120;  DUTY[ 6] <= 105;  DUTY[ 5] <=  90;  DUTY[ 4] <=  75;
      DUTY[ 3] <=  60;  DUTY[ 2] <=  45;  DUTY[ 1] <=  30;  DUTY[ 0] <=  15;
   end


   always @ (posedge CLK) begin
      if (&CNT) begin
	 DUTY[15] <= DUTY[ 0]; DUTY[14] <= DUTY[15];
	 DUTY[13] <= DUTY[14]; DUTY[12] <= DUTY[13];
	 DUTY[11] <= DUTY[12]; DUTY[10] <= DUTY[11];
	 DUTY[ 9] <= DUTY[10]; DUTY[ 8] <= DUTY[ 9];
	 DUTY[ 7] <= DUTY[ 8]; DUTY[ 6] <= DUTY[ 7];
	 DUTY[ 5] <= DUTY[ 6]; DUTY[ 4] <= DUTY[ 5];
	 DUTY[ 3] <= DUTY[ 4]; DUTY[ 2] <= DUTY[ 3];
	 DUTY[ 1] <= DUTY[ 2]; DUTY[ 0] <= DUTY[ 1];
      end
   end
   
   duty_ctrl dc15 ( .CLK(CLK), .DUTY(DUTY[15]), .Q(LED[15]) );
   duty_ctrl dc14 ( .CLK(CLK), .DUTY(DUTY[14]), .Q(LED[14]) );
   duty_ctrl dc13 ( .CLK(CLK), .DUTY(DUTY[13]), .Q(LED[13]) );
   duty_ctrl dc12 ( .CLK(CLK), .DUTY(DUTY[12]), .Q(LED[12]) );

   duty_ctrl dc11 ( .CLK(CLK), .DUTY(DUTY[11]), .Q(LED[11]) );
   duty_ctrl dc10 ( .CLK(CLK), .DUTY(DUTY[10]), .Q(LED[10]) );
   duty_ctrl dc09 ( .CLK(CLK), .DUTY(DUTY[ 9]), .Q(LED[ 9]) );
   duty_ctrl dc08 ( .CLK(CLK), .DUTY(DUTY[ 8]), .Q(LED[ 8]) );

   duty_ctrl dc07 ( .CLK(CLK), .DUTY(DUTY[ 7]), .Q(LED[ 7]) );
   duty_ctrl dc06 ( .CLK(CLK), .DUTY(DUTY[ 6]), .Q(LED[ 6]) );
   duty_ctrl dc05 ( .CLK(CLK), .DUTY(DUTY[ 5]), .Q(LED[ 5]) );
   duty_ctrl dc04 ( .CLK(CLK), .DUTY(DUTY[ 4]), .Q(LED[ 4]) );

   duty_ctrl dc03 ( .CLK(CLK), .DUTY(DUTY[ 3]), .Q(LED[3]) );
   duty_ctrl dc02 ( .CLK(CLK), .DUTY(DUTY[ 2]), .Q(LED[2]) );
   duty_ctrl dc01 ( .CLK(CLK), .DUTY(DUTY[ 1]), .Q(LED[1]) );
   duty_ctrl dc00 ( .CLK(CLK), .DUTY(DUTY[ 0]), .Q(LED[0]) );
   
   
endmodule
